/*
	Engineering Test: Boggle Solver
	Gordon Lee

*******************************************************************************

	@filename	cApplication.h

	@description
		Class for Application that handles the window and run loop

	@author		Gordon Lee
*/

#ifndef C_APPLICATION_H
#define C_APPLICATION_H
#pragma once

// Includes
#define WIN32_LEAN_AND_MEAN
#undef UNICODE
#undef _UNICODE
#include <windows.h>
#include <string>
#include "cTimer.h"
#include "cDictionary.h"

// Namespace
namespace Test
{
////////////////////////////////////////////////////////////////////////////////
// Defines
#define SCREEN_WIDTH	800
#define SCREEN_HEIGHT	600
#define BUF_LEN			256
#define DICTIONARY_FILE	"dictionary.txt"

class cApplication
{
public:
	static cApplication *	Instance(void);
	static cApplication *	Create(void);
	static void				Destroy(void);
	bool					Initialize(HINSTANCE _hInstance, std::string _title);

	bool					OpenConsoleWindow(void);
	bool					CloseConsoleWindow(void);

	void					Run(void);
	LRESULT					AppWndProc(HWND _hWnd, UINT _msg, WPARAM _wParam, LPARAM _lParam);

private:
							cApplication();
							~cApplication();

	// Data members
	std::string m_winClassName;
	HINSTANCE m_hInstance;
	HWND m_hWnd;
	HWND m_hInputWindow;
	HWND m_hOutputWindow;
	HWND m_hOutputButton;
	HWND m_hMinimumLettersComboBox;
	bool m_bConsoleOpen;
	cTimer m_debugTimer;
	cDictionary *m_dictionary;
};
////////////////////////////////////////////////////////////////////////////////
// Global define
#define g_pcApplication cApplication::Instance()
}

#endif
