/*
	Engineering Test: Boggle Solver
	Gordon Lee

*******************************************************************************

	@filename	cMemoryManager.h

	@description
		Class for a memory manager for pooling and handling memory.

	@author		Gordon Lee
*/

#ifndef C_MEMORYMANAGER_H
#define C_MEMORYMANAGER_H
#pragma once

// Includes
#include <map>
#include "cMemoryPool.h"

namespace Test
{
////////////////////////////////////////////////////////////////////////////////
// forward declarations
class cMemoryPool;

// Typedefs
typedef std::map<size_t, cMemoryPool *> MemoryPoolMap;

class cMemoryManager
{
public:
	static cMemoryManager *		Instance(void);
	static cMemoryManager *		Create(void);
	static void					Destroy(void);

	void *						Allocate(size_t _objectSize);
	void						Free(size_t _objectSize, void *_object);
	void						Clear(void);

private:
								cMemoryManager();
								~cMemoryManager();

	MemoryPoolMap::iterator		CreateMemoryPool(size_t _objectSize);

	// data members
	MemoryPoolMap m_memoryPoolMap;

	unsigned m_numAllocations;
	unsigned m_numFrees;
	unsigned m_numPools;
	unsigned m_numUsedMemory;
	unsigned m_maxMemoryUsed;
	unsigned m_totalMemoryAllocated;
};
////////////////////////////////////////////////////////////////////////////////
// Global define
#define g_pcMemoryManager cMemoryManager::Instance()
}

#endif
