/*
	Engineering Test: Boggle Solver
	Gordon Lee

*******************************************************************************

	@filename	cTrie.cpp

	@description
		Class for a trie (prefix tree) graph

	@author		Gordon Lee
*/

// Includes
#include "cTrie.h"
#include <string>
#include <stdio.h>
#include <ctype.h>
#include "cMemoryManager.h"

// Namespace
namespace Test
{
////////////////////////////////////////////////////////////////////////////////
/**
	Overload New

	@param size_t _size

	@return void *
*/
void *sTrieNode::operator new(size_t _size)
{
	return g_pcMemoryManager->Allocate(_size);
}

/**
	Overload Delete

	@param void *_p
	@param size_t _size
*/
void sTrieNode::operator delete(void *_p, size_t _size)
{
	g_pcMemoryManager->Free(_size, _p);
}

/**
	Constructor
*/
cTrie::cTrie()
: m_size(1)
{
	m_head = new sTrieNode;
	m_head->m_char = 0;
	m_head->m_child = NULL;
	m_head->m_sibling = NULL;
	m_head->m_bEOW = false;
}

/**
	Deconstructor
*/
cTrie::~cTrie()
{
	Delete(m_head);
}

/**
	Head

	@return sTrieNode *
*/
sTrieNode *cTrie::Head(void)
{
	return m_head;
}

/**
	Recursive Insert Word

	@param char *_word
	@param int _wordIndex
	@param sTrieNode *_nodeRef
*/
void cTrie::Insert(char *_word, int _wordIndex, sTrieNode *_nodeRef)
{
	sTrieNode *_temp = _nodeRef->m_child;
	size_t _wordLength = strlen(_word) - 1;
	char _fLetterFound = LETTER_SEARCHING;

	// Exit condition
	if (_wordIndex == _wordLength)
		return;
	
	while (_temp && _fLetterFound == LETTER_SEARCHING)
	{
		// Try to find letter within siblings
		if (_temp->m_char != tolower(_word[_wordIndex]))
		{
			if (_temp->m_sibling != NULL)
				_temp = _temp->m_sibling;
			else
				_fLetterFound = LETTER_NOTFOUND;
		}
		// Found letter within siblings
		else
			_fLetterFound = LETTER_FOUND;
	}
	// Not found, create a new node
	if (_fLetterFound == LETTER_SEARCHING || _fLetterFound == LETTER_NOTFOUND)
	{
		// Create a new node
		sTrieNode *_node = new sTrieNode;
		_node->m_char = tolower(_word[_wordIndex]);
		_node->m_child = NULL;
		_node->m_sibling = NULL;
		// End Of Word flag
		if (_wordIndex == _wordLength - 1)
			_node->m_bEOW = true;
		else
			_node ->m_bEOW = false;

		++m_size;

		// Insert as sibling
		if (_nodeRef->m_child != NULL)
		{
			_temp->m_sibling = _node;

			// Go to next letter
			Insert(_word, _wordIndex + 1, _node);
		}
		// Insert as child
		else
		{
			_nodeRef->m_child = _node;

			// Go to next letter
			Insert(_word, _wordIndex + 1, _node);
		}
	}
	// Found, go to next letter
	else
		Insert(_word, _wordIndex + 1, _temp);
}

/**
	Delete

	@param sTrieNode *_nodeRef
*/
void cTrie::Delete(sTrieNode *_nodeRef)
{
	// Exit condition
	if (!_nodeRef)
		return;

	Delete(_nodeRef->m_sibling);
	Delete(_nodeRef->m_child);

	--m_size;
	delete _nodeRef;
}
////////////////////////////////////////////////////////////////////////////////
}
